#include "stats.h"
#include "application_class.h"

stats::stats() : fps_(nullptr), drawcalls_(0)
{
}

stats::~stats()
{
    fps_ = nullptr;
}

bool stats::initialize(fps_class* fps, application_class* app)
{
    if (!fps) {
        Logger::Get().Log("FPS pointer is null", __FILE__, __LINE__, Logger::LogLevel::Error);
        return false;
    }

    fps_ = fps;
    drawcalls_ = 0;
    app_ = app;

    // call update() in a separate thread
    std::thread updateThread(&stats::update, this);
    updateThread.detach(); // Detach the thread to run independently
    Logger::Get().Log("Stats initialized successfully", __FILE__, __LINE__, Logger::LogLevel::Initialize);
    
    return true;
}

void stats::update()
{
    while (true)
    {
        // call update_stats() every second
        std::this_thread::sleep_for(std::chrono::seconds(1));
        update_stats();
    }
}

void stats::update_stats()
{
    object_vec_ = app_->get_kobjects();
    cubes_vec_ = app_->get_cubes();
    terrain_chunk_vec_ = app_->get_terrain_cubes();

    current_fps_ = get_current_fps();
    min_fps_ = get_min_fps();
    max_fps_ = get_max_fps();
    drawcalls_ = get_draw_calls();
    total_vertex_count_ = get_total_vertex_count();
    total_triangle_count_ = get_total_triangle_count();
    visible_triangle_count_ = get_visible_triangle_count();
    current_frame_time_ = get_frame_time();
}

int stats::get_total_vertex_count() const
{
    int totalVertices = 0;
    for (const auto& obj : object_vec_)
    {
        if (obj)
        {
            totalVertices += obj->GetVertexCount();
        }
    }
    // Ajoutez le nombre de sommets pour les cubes
    for (const auto& cube : cubes_vec_)
    {
        if (cube)
        {
            totalVertices += cube->GetVertexCount();
        }
    }
    // Ajoutez le nombre de sommets pour le terrain
    for (const auto& chunk : terrain_chunk_vec_)
    {
        if (chunk)
        {
            totalVertices += chunk->GetVertexCount();
        }
    }
    return totalVertices;
}

int stats::get_total_triangle_count() const
{
    int totalTriangles = 0;
    for (const auto& obj : object_vec_)
    {
        if (obj)
        {
            // Dans une topologie de liste de triangles, chaque triangle utilise 3 indices
            totalTriangles += obj->GetIndexCount() / 3;
        }
    }
    // Ajoutez le nombre de triangles pour les cubes
    for (const auto& cube : cubes_vec_)
    {
        if (cube)
        {
            totalTriangles += cube->GetIndexCount() / 3;
        }
    }
    // Ajoutez le nombre de triangles pour le terrain
    for (const auto& chunk : terrain_chunk_vec_)
    {
        if (chunk)
        {
            totalTriangles += chunk->GetIndexCount() / 3;
        }
    }
    return totalTriangles;
}

int stats::get_visible_triangle_count() const
{
    int visibleTriangles = 0;
    for (const auto& obj : object_vec_)
    {
        if (obj && obj->IsVisible())
        {
            visibleTriangles += obj->GetIndexCount() / 3;
        }
    }
    // Ajoutez le nombre de triangles visibles pour les cubes
    for (const auto& cube : cubes_vec_)
    {
        if (cube && cube->IsVisible())
        {
            visibleTriangles += cube->GetIndexCount() / 3;
        }
    }
    // Ajoutez le nombre de triangles visibles pour le terrain
    for (const auto& chunk : terrain_chunk_vec_)
    {
        if (chunk && chunk->IsVisible())
        {
            visibleTriangles += chunk->GetIndexCount() / 3;
        }
    }
    return visibleTriangles;
}